import pykraken
from .exceptions import RequiredParameterError
from .convert import commasep, parseOTime


def kprivate_balance(client):
    c = client._post("/0/private/Balance")
    return c['result']


def kprivate_tradebalance(client, aclass='currency', asset='ZUSD'):
    params = {}
    if aclass:
        params['aclass'] = aclass
    if asset:
        params['asset'] = asset
    c = client._post("/0/private/TradeBalance", params)
    return c['result']


def kprivate_openorders(client, trades=False, userref=None):
    params = {}
    if trades:
        params['trades'] = trades
    if userref:
        params['userref'] = userref
    c = client._post("/0/private/OpenOrders", params)
    return c['result']


def kprivate_closedorders(client, trades=False, userref=None, start=None, end=None, ofs=None, closetime='both'):
    params = {}
    if trades:
        params['trades'] = trades
    if userref:
        params['userref'] = userref
    if start:
        params['start'] = start
    if end:
        params['end'] = end
    if ofs:
        params['ofs'] = ofs
    if closetime:
        params['closetime'] = closetime

    c = client._post("/0/private/ClosedOrders", params)
    return c['result']


def kprivate_queryorders(client, trades=False, userref=None, txid=None):
    params = {}
    if trades:
        params['trades'] = trades
    if userref:
        params['userref'] = userref
    if txid:
        params['txid'] = commasep(txid)

    c = client._post("/0/private/QueryOrders", params)
    return c['result']


def kprivate_tradeshistory(client, typet=None, trades=False, start=None, end=None, ofs=None):
    params = {}
    if typet and typet in ['all', 'any position', 'closed position', 'closing position', 'no position']:
        # using typet variable as type is reserved, but it need to be type in the params dictionnary
        params['type'] = typet
    if trades:
        params['trades'] = trades
    if start:
        params['start'] = start
    if end:
        params['end'] = end
    if ofs:
        params['ofs'] = ofs

    c = client._post("/0/private/TradesHistory", params)
    return c['result']


def kprivate_querytrades(client, txid=None, trades=False):
    params = {}
    if txid and len(txid) <= 20 and isinstance(txid, list):
        params['txid'] = commasep(txid)
    else:
        raise pykraken.exceptions.RequiredParameterError('no txid found')
    if trades:
        params['trades'] = trades
    c = client._post("/0/private/QueryTrades", params)
    return c['result']


def kprivate_openpositions(client, txid=None, docalcs=False):
    params = {}
    if txid and isinstance(txid, list):
        params['txid'] = commasep(txid)
    if docalcs:
        params['docalcs'] = docalcs

    c = client._post("/0/private/OpenPositions", params)
    return c['result']


def kprivate_ledgers(client, aclass='currency', asset='all', typet='all', start=None, end=None, ofs=None):
    params = {}
    if aclass:
        params['aclass'] = aclass
    if asset:
        params['asset'] = asset
    if typet:
        params['type'] = typet
    if start:
        params['start'] = start
    if end:
        params['end'] = end
    if ofs:
        params['ofs'] = ofs

    c = client._post("/0/private/Ledgers")
    return c['result']


def kprivate_queryledgers(client, id=None):
    params = {}
    if id and len(id) <= 20 and isinstance(id, list):
        params['id'] = commasep(id)
    else:
        raise pykraken.exceptions.BadParamterError('error in ids')
    c = client._post("/0/private/QueryLedgers", params)
    return c['result']


def kprivate_tradevolume(client, pair=None, feeinfo=None):
    params = {}
    if pair:
        params['pair'] = commasep(pair)
    if feeinfo:
        params['fee-info'] = feeinfo

    c = client._post("/0/private/TradeVolume")
    return c['result']


ORDER_TYPES_0 = ['market']
ORDER_TYPES_1 = ['limit', 'stop-loss', 'take-profit', 'trailing-stop']
ORDER_TYPES_2 = ['stop-loss-profit', 'stop-loss-profit-limit', 'stop-loss-limit', 'take-profit-limit',
                 'trailing-stop-limit', 'stop-loss-and-limit']
ORDER_FLAGS = ['viqc', 'fcib', 'fciq', 'nompp', 'post']


def kprivate_addorder(client, pair=None, typeo=None, ordertype=None, price=None, price2=None, volume=None,
                      leverage=None, oflags=None,
                      starttm=None, expiretm=None, userref=None, validate=None):
    params = {}
    if pair:
        params['pair'] = pair
    else:
        raise RequiredParameterError('pair')
    if typeo and typeo in ['buy', 'sell']:
        params['type'] = typeo
    else:
        raise RequiredParameterError('typeo')
    if ordertype and ordertype in (ORDER_TYPES_0 + ORDER_TYPES_1 + ORDER_TYPES_2):
        params['ordertype'] = ordertype
    else:
        raise RequiredParameterError('ordertype')

    if ordertype in ORDER_TYPES_0:
        if price:
            raise pykraken.exceptions.BadParamterError('if price is set, ordertype cant be at market')
    elif ordertype in ORDER_TYPES_1:
        if price:
            params['price'] = price
        else:
            raise pykraken.exceptions.RequiredParameterError(
                'price required for this order type: {}'.format(ordertype))
    elif ordertype in ORDER_TYPES_2:
        if price and price2:
            params['price'] = price
            params['price2'] = price2
        else:
            raise pykraken.exceptions.RequiredParameterError(
                'price and price2 required for this order type: {}'.format(ordertype))
    else:
        raise pykraken.exceptions.BadParamterError(
            'ordertype: {} not allowed, it should be in {} or {} or {}'.format(ordertype, ORDER_TYPES_0, ORDER_TYPES_1,
                                                                               ORDER_TYPES_2))
    if volume:
        params['volume'] = volume
    else:
        raise pykraken.exceptions.RequiredParameterError('volume is required')

    if leverage:
        params['leverage'] = leverage

    if oflags:
        params['oflags'] = commasep(oflags)
    if starttm:
        params['starttm'] = parseOTime(starttm)
    if expiretm:
        params['expiretm'] = parseOTime(expiretm)
    if userref:
        params['userref'] = userref
    if validate:
        params['validate'] = validate

    c = client._post("/0/private/AddOrder", params)
    return c['result']


def kprivate_cancelorder(client, txid=None):
    params = {}
    if txid:
        params['txid'] = txid
    else:
        raise pykraken.exceptions.RequiredParameterError('transaction id required')

    c = client._post("/0/private/CancelOrder", params)
    return c['result']


def kprivate_depositmethods():
    pass




import pytest

import pykraken
from pykraken.exceptions import RequiredParameterError
from pykraken.config import PROXY, API_KEY, PRIVATE_KEY


def test_balance():
    client = pykraken.Client(key=API_KEY, private_key=PRIVATE_KEY, requests_kwargs=PROXY)
    t = client.kprivate_balance()
    assert 'XXBT' in t.keys()


def test_trade_balance():
    client = pykraken.Client(key=API_KEY, private_key=PRIVATE_KEY, requests_kwargs=PROXY)
    t = client.kprivate_tradebalance()
    # print(t.keys())
    # ml should be in this list, dunno why on my account it's not, maybe because i don't have margin yet
    tbkeys = ['eb', 'tb', 'm', 'n', 'c', 'v', 'e', 'mf']
    tbbool = [k in t.keys() for k in tbkeys]
    assert all(tbbool)


def test_open_orders():
    client = pykraken.Client(key=API_KEY, private_key=PRIVATE_KEY, requests_kwargs=PROXY)
    t = client.kprivate_openorders(trades=False)
    assert 'open' in t.keys()


def test_closed_orders():
    client = pykraken.Client(key=API_KEY, private_key=PRIVATE_KEY, requests_kwargs=PROXY)
    t = client.kprivate_closedorders(trades=False)
    assert 'closed' in t.keys()


def test_tradesHistory_queryTrades():
    client = pykraken.Client(key=API_KEY, private_key=PRIVATE_KEY, requests_kwargs=PROXY)
    t = client.kprivate_tradeshistory(trades=False)
    assert 'count' in t.keys()
    txidexample = [list(t['trades'].keys())][0]
    t1 = client.kprivate_querytrades(txid=txidexample)
    assert (txidexample[0] in t1.keys())


def test_openPositions():
    client = pykraken.Client(key=API_KEY, private_key=PRIVATE_KEY, requests_kwargs=PROXY)
    t = client.kprivate_openpositions()
    # TODO find a better test this one tests nothing
    assert type(t) == dict


def test_getLedgers_and_query():
    client = pykraken.Client(key=API_KEY, private_key=PRIVATE_KEY, requests_kwargs=PROXY)
    t = client.kprivate_ledgers()
    # TODO find a better test
    assert 'count' in t.keys()
    ledgeriidlist = [list(t['ledger'].keys())[0]]
    t1 = client.kprivate_queryledgers(id=ledgeriidlist)
    # TODO find a better test
    assert ledgeriidlist[0] in t1.keys()


def test_tradeVolume():
    client = pykraken.Client(key=API_KEY, private_key=PRIVATE_KEY, requests_kwargs=PROXY)
    t = client.kprivate_tradevolume()
    # TODO find a better test
    assert 'currency' in t.keys()


def test_order_required_pair():
    with pytest.raises(RequiredParameterError):
        client = pykraken.Client(key=API_KEY, private_key=PRIVATE_KEY, requests_kwargs=PROXY)
        t = client.kprivate_addorder()


def test_addAndCancelOrder():
    # add validate=True just to enter false orders
    client = pykraken.Client(key=API_KEY, private_key=PRIVATE_KEY, requests_kwargs=PROXY)
    t = client.kprivate_addorder(pair='XETHZEUR', typeo='buy', ordertype='limit', price='+5.0', volume=0.01, validate=True)
    assert 'descr' in t.keys()
    # referral_tid = t['txid']
    # openorderidList = client.kprivate_openorders()
    # assert referral_tid in openorderidList['open'].keys()
    # cancel = client.kprivate_cancelorder(referral_tid)
    # assert 'count' in cancel.keys()



import time
import pykraken
import pytest
from pykraken.config import PROXY, API_KEY, PRIVATE_KEY


def test_no_api_key():
    with pytest.raises(Exception):
        client = pykraken.Client()

def test_server_time():
    client = pykraken.Client(API_KEY, PRIVATE_KEY, requests_kwargs=PROXY)
    utcnow = time.time()
    t = client.kpublic_time()
    # t_compare = datetime.strptime(t[1], '%a, %d %b %y %H:%M:%S +0000')
    t_compare = t[0]
    print("t_compare: {} utcnow: {}".format(t_compare, utcnow))
    delta = t_compare - utcnow
    assert abs(delta)<= 10

def test_assets_asset_parameter():
    client = pykraken.Client(API_KEY, PRIVATE_KEY, requests_kwargs=PROXY)
    t = client.kpublic_assets(asset=['XETH'])
    assert u'XETH' in t.keys()

def test_assets_aclass_parameter():
    with pytest.raises(pykraken.exceptions.BadParamterError):
        client = pykraken.Client(API_KEY, PRIVATE_KEY, requests_kwargs=PROXY)
        t = client.kpublic_assets(aclass='mouahahah bad parameter')

def test_assetpairs():
    client = pykraken.Client(API_KEY, PRIVATE_KEY, requests_kwargs=PROXY)
    t = client.kpublic_assetpairs()
    # TODO: find a better test
    assert 'XXBTZUSD' in t.keys()

def test_ticker():
    client = pykraken.Client(API_KEY, PRIVATE_KEY, requests_kwargs=PROXY)
    t = client.kpublic_ticker(pair=['XETHXXBT'])
    print(t)
    assert 'XETHXXBT' in t.keys()

def test_OHLC():
    client = pykraken.Client(API_KEY, PRIVATE_KEY, requests_kwargs=PROXY)
    t = client.kpublic_ohlc(pair=['XETHXXBT'])
    print(t)
    assert 'XETHXXBT' in t.keys()

def test_depth():
    client = pykraken.Client(API_KEY, PRIVATE_KEY, requests_kwargs=PROXY)
    t = client.kpublic_depth(pair=['XETHXXBT'])
    print(t)
    assert 'XETHXXBT' in t.keys()

def test_trades():
    client = pykraken.Client(API_KEY, PRIVATE_KEY, requests_kwargs=PROXY)
    t = client.kpublic_trades(pair=['XETHXXBT'])
    print(t)
    assert 'XETHXXBT' in t.keys()

def test_spread():
    client = pykraken.Client(API_KEY, PRIVATE_KEY, requests_kwargs=PROXY)
    t = client.kpublic_spread(pair=['XETHXXBT'])
    print(t)
    assert 'XETHXXBT' in t.keys()



"""
Core client functionality, common across all API requests (including performing HTTP requests).
"""

import base64
import collections
from datetime import datetime
from datetime import timedelta
import hashlib
import hmac

import requests
import random
import time

import pykraken
from .exceptions import _RetriableRequest, ApiError

try:  # Python 3
    from urllib.parse import urlencode
except ImportError:  # Python 2
    from urllib import urlencode

_USER_AGENT = "pykraken {} (https://github.com/euri10/pykraken)".format(pykraken.__version__)
_DEFAULT_BASE_URL = "https://api.kraken.com"

_RETRIABLE_STATUSES = set([500, 503, 504])


class Client(object):
    """Performs requests to the kraken API."""

    def __init__(self, key=None, private_key=None, timeout=None, connect_timeout=None, read_timeout=None,
                 retry_timeout=60, requests_kwargs=None,
                 queries_per_second=10):
        """
        :param key: API key.
        :type key: string

        :param timeout: Combined connect and read timeout for HTTP requests, in
            seconds. Specify "None" for no timeout.
        :type timeout: int

        :param connect_timeout: Connection timeout for HTTP requests, in
            seconds. You should specify read_timeout in addition to this option.
            Note that this requires requests >= 2.4.0.
        :type connect_timeout: int

        :param read_timeout: Read timeout for HTTP requests, in
            seconds. You should specify connect_timeout in addition to this
            option. Note that this requires requests >= 2.4.0.
        :type read_timeout: int

        :param retry_timeout: Timeout across multiple retriable requests, in
            seconds.
        :type retry_timeout: int

        :param queries_per_second: Number of queries per second permitted.
            If the rate limit is reached, the client will sleep for the
            appropriate amount of time before it runs the current query.
        :type queries_per_second: int

        :raises ValueError: when either credentials are missing, incomplete
            or invalid.
        :raises NotImplementedError: if connect_timeout and read_timeout are
            used with a version of requests prior to 2.4.0.

        :param requests_kwargs: Extra keyword arguments for the requests
            library, which among other things allow for proxy auth to be
            implemented. See the official requests docs for more info:
            http://docs.python-requests.org/en/latest/api/#main-interface
        :type requests_kwargs: dict

        """
        if not key:
            raise ValueError("Must provide API key when creating client.")

        self.key = key
        self.private_key = private_key

        if timeout and (connect_timeout or read_timeout):
            raise ValueError("Specify either timeout, or connect_timeout " +
                             "and read_timeout")

        if connect_timeout and read_timeout:
            # Check that the version of requests is >= 2.4.0
            chunks = requests.__version__.split(".")
            if chunks[0] < 2 or (chunks[0] == 2 and chunks[1] < 4):
                raise NotImplementedError("Connect/Read timeouts require "
                                          "requests v2.4.0 or higher")
            self.timeout = (connect_timeout, read_timeout)
        else:
            self.timeout = timeout

        self.retry_timeout = timedelta(seconds=retry_timeout)
        self.requests_kwargs = requests_kwargs or {}
        self.requests_kwargs.update({
            "headers": {"User-Agent": _USER_AGENT, "API-Key": self.key},
            "timeout": self.timeout,
            "verify": True,  # NOTE(cbro): verify SSL certs.
        })

        self.queries_per_second = queries_per_second
        self.sent_times = collections.deque("", queries_per_second)

    def _post(self, url, params={}, first_request_time=None, retry_counter=0,
              base_url=_DEFAULT_BASE_URL, accepts_clientid=True,
              extract_body=None, requests_kwargs=None):

        if not first_request_time:
            first_request_time = datetime.now()

        elapsed = datetime.now() - first_request_time
        if elapsed > self.retry_timeout:
            raise pykraken.exceptions.Timeout()

        if retry_counter > 0:
            # 0.5 * (1.5 ^ i) is an increased sleep time of 1.5x per iteration,
            # starting at 0.5s when retry_counter=0. The first retry will occur
            # at 1, so subtract that first.
            delay_seconds = 0.5 * 1.5 ** (retry_counter - 1)

            # Jitter this value by 50% and pause.
            time.sleep(delay_seconds * (random.random() + 0.5))

        # Unicode-objects must be encoded before hashing
        # "API-Sign = Message signature using HMAC-SHA512 of (URI path + SHA256(nonce + POST data)) and base64 decoded secret API key"
        params['nonce'] = int(1000 * time.time())

        postdata = urlencode(params)

        # Unicode-objects must be encoded before hashing
        encoded = (str(params['nonce']) + postdata).encode()
        message = url.encode() + hashlib.sha256(encoded).digest()

        signature = hmac.new(base64.b64decode(self.private_key), message, hashlib.sha512)
        sigdigest = base64.b64encode(signature.digest())

        self.requests_kwargs.update({
            "headers": {"User-Agent": _USER_AGENT, "API-Key": self.key, "API-Sign": sigdigest.decode()},
            "timeout": self.timeout,
            "verify": True,  # NOTE(cbro): verify SSL certs.
        })

        # Default to the client-level self.requests_kwargs, with method-level
        # requests_kwargs arg overriding.
        requests_kwargs = dict(self.requests_kwargs, **(requests_kwargs or {}))
        try:
            resp = requests.post(base_url + url, data=params, **requests_kwargs)
        except requests.exceptions.Timeout:
            raise pykraken.exceptions.Timeout()
        except Exception as e:
            raise pykraken.exceptions.TransportError(e)

        if resp.status_code in _RETRIABLE_STATUSES:
            # Retry request.
            return self._post(url, params, first_request_time, retry_counter + 1,
                              base_url, accepts_clientid, extract_body)

        # Check if the time of the nth previous query (where n is queries_per_second)
        # is under a second ago - if so, sleep for the difference.
        if self.sent_times and len(self.sent_times) == self.queries_per_second:
            elapsed_since_earliest = time.time() - self.sent_times[0]
            if elapsed_since_earliest < 1:
                time.sleep(1 - elapsed_since_earliest)

        try:
            if extract_body:
                result = extract_body(resp)
            else:
                result = self._get_body(resp)
            self.sent_times.append(time.time())
            return result
        except _RetriableRequest:
            # Retry request.
            return self._post(url, params, first_request_time, retry_counter + 1,
                              base_url, accepts_clientid, extract_body)

    def _get_body(self, resp):
        if resp.status_code != 200:
            raise pykraken.exceptions.HTTPError(resp.status_code)

        body = resp.json()

        if len(body["error"]):
            raise ApiError(resp.status_code, message=body["error"])
        else:
            return body

# public market data https://www.kraken.com/help/api#public-market-data
from .kpublic import kpublic_time
from .kpublic import kpublic_assets
from .kpublic import kpublic_assetpairs
from .kpublic import kpublic_ticker
from .kpublic import kpublic_ohlc
from .kpublic import kpublic_depth
from .kpublic import kpublic_trades
from .kpublic import kpublic_spread

# private user data https://www.kraken.com/help/api#private-user-data
from .kprivate import kprivate_balance
from .kprivate import kprivate_tradebalance
from .kprivate import kprivate_openorders
from .kprivate import kprivate_closedorders
from .kprivate import kprivate_tradeshistory
from .kprivate import kprivate_querytrades
from .kprivate import kprivate_openpositions
from .kprivate import kprivate_ledgers
from .kprivate import kprivate_queryledgers
from .kprivate import kprivate_tradevolume

# private user trading https://www.kraken.com/help/api#private-user-trading
from .kprivate import kprivate_addorder
from .kprivate import kprivate_cancelorder

Client.kpublic_time = kpublic_time
Client.kpublic_assets = kpublic_assets
Client.kpublic_assetpairs = kpublic_assetpairs
Client.kpublic_ticker = kpublic_ticker
Client.kpublic_ohlc = kpublic_ohlc
Client.kpublic_depth = kpublic_depth
Client.kpublic_trades = kpublic_trades
Client.kpublic_spread = kpublic_spread

Client.kprivate_balance = kprivate_balance
Client.kprivate_tradebalance = kprivate_tradebalance
Client.kprivate_openorders = kprivate_openorders
Client.kprivate_closedorders = kprivate_closedorders
Client.kprivate_tradeshistory = kprivate_tradeshistory
Client.kprivate_querytrades = kprivate_querytrades
Client.kprivate_openpositions = kprivate_openpositions
Client.kprivate_ledgers = kprivate_ledgers
Client.kprivate_queryledgers = kprivate_queryledgers
Client.kprivate_tradevolume = kprivate_tradevolume
Client.kprivate_addorder = kprivate_addorder
Client.kprivate_cancelorder = kprivate_cancelorder


def sign_hmac(secret, payload):
    """Returns a base64-encoded HMAC-SHA1 signature of a given string.

    :param secret: The key used for the signature, base64 encoded.
    :type secret: string

    :param payload: The payload to sign.
    :type payload: string

    :rtype: string
    """
    payload = payload.encode('ascii', 'strict')
    secret = secret.encode('ascii', 'strict')
    sig = hmac.new(base64.urlsafe_b64decode(secret), payload, hashlib.sha1)
    out = base64.urlsafe_b64encode(sig.digest())
    return out.decode('utf-8')


def urlencode_params(params):
    """URL encodes the parameters.

    :param params: The parameters
    :type params: list of key/value tuples.

    :rtype: string
    """
    # urlencode does not handle unicode strings in Python 2.
    # Firstly, normalize the values so they get encoded correctly.
    params = [(key, normalize_for_urlencode(val)) for key, val in params]
    # Secondly, unquote unreserved chars which are incorrectly quoted
    # by urllib.urlencode, causing invalid auth signatures. See GH #72
    # for more info.
    return requests.utils.unquote_unreserved(urlencode(params))


try:
    unicode


    # NOTE(cbro): `unicode` was removed in Python 3. In Python 3, NameError is
    # raised here, and caught below.

    def normalize_for_urlencode(value):
        """(Python 2) Converts the value to a `str` (raw bytes)."""
        if isinstance(value, unicode):
            return value.encode('utf8')

        if isinstance(value, str):
            return value

        return normalize_for_urlencode(str(value))

except NameError:
    def normalize_for_urlencode(value):
        """(Python 3) No-op."""
        # urlencode in Python 3 handles all the types we are passing it.
        return value



import pykraken
from .exceptions import BadParamterError
from .convert import commasep


def kpublic_time(client):
    """
    Returns the servers' time
    :param client: the client
    :return: a tuple (unixtime =  as unix timestamp, rfc1123 = as RFC 1123 time format)
    """
    c = client._post("/0/public/Time")
    return c['result']['unixtime'], c['result']['rfc1123']


def kpublic_assets(client, info='info', aclass=None, asset=None):
    """
    Returns array of asset names and their info
    :param client: the client
    :param info: info to retrieve (optional), 'info' by default (doc is weird about that, optionnal but only one option...)
    :param aclass: asset class (optional), 'currency' is the default (same weirdness in doc)
    :param asset: comma delimited list of assets to get info on (optional.  default = all for given asset class)
    :return: <asset_name> = asset name
                altname = alternate name
                aclass = asset class
                decimals = scaling decimal places for record keeping
                display_decimals = scaling decimal places for output display
    """
    params = {}
    if info:
        params['info'] = info
    if asset:
        params['asset'] = commasep(asset)
    if aclass:
        if aclass is not "currency":
            raise BadParamterError('aclass should be currency')
    c = client._post("/0/public/Assets", params)
    return c['result']


def kpublic_assetpairs(client, info='info', pair=None):
    """
    Returns an array of pair names and some info about it
    :param client: the client
    :param info: pinfo to retrieve (optional), 'info' by default (doc is weird about that, optionnal but only one option...)
    :param pair:  comma delimited list of asset pairs to get info on (optional.  default = all)
    :return: <pair_name> = pair name
        altname = alternate pair name
        aclass_base = asset class of base component
        base = asset id of base component
        aclass_quote = asset class of quote component
        quote = asset id of quote component
        lot = volume lot size
        pair_decimals = scaling decimal places for pair
        lot_decimals = scaling decimal places for volume
        lot_multiplier = amount to multiply lot volume by to get currency volume
        leverage_buy = array of leverage amounts available when buying
        leverage_sell = array of leverage amounts available when selling
        fees = fee schedule array in [volume, percent fee] tuples
        fees_maker = maker fee schedule array in [volume, percent fee] tuples (if on maker/taker)
        fee_volume_currency = volume discount currency
        margin_call = margin call level
        margin_stop = stop-out/liquidation margin level
    """
    params = {}
    if info not in ['info', 'leverage', 'fees', 'margin']:
        raise pykraken.exceptions.BadParamterError()
    else:
        params['info'] = info
    if pair:
        params['pair'] = commasep(pair)

    c = client._post("/0/public/AssetPairs", params)
    return c['result']


def kpublic_ticker(client, pair=None):
    """
    Returns an array of pair names and their ticker info
    :param client: the client
    :param pair: comma delimited list of asset pairs to get info on
    :return: <pair_name> = pair name
        a = ask array(<price>, <whole lot volume>, <lot volume>),
        b = bid array(<price>, <whole lot volume>, <lot volume>),
        c = last trade closed array(<price>, <lot volume>),
        v = volume array(<today>, <last 24 hours>),
        p = volume weighted average price array(<today>, <last 24 hours>),
        t = number of trades array(<today>, <last 24 hours>),
        l = low array(<today>, <last 24 hours>),
        h = high array(<today>, <last 24 hours>),
        o = today's opening price
    """
    params = {}
    if pair:
        params['pair'] = commasep(pair)
    else:
        raise pykraken.exceptions.BadParamterError()
    c = client._post("/0/public/Ticker", params)
    return c['result']


def kpublic_ohlc(client, pair=None, interval=1, since=None):
    """
    Returns array of pair name and OHLC data
    :param client: the client
    :param pair: asset pair to get OHLC data for
    :param interval: time frame interval in minutes (optional): 1 (default), 5, 15, 30, 60, 240, 1440, 10080, 21600
    :param since: return committed OHLC data since given id (optional.  exclusive)
    :return: <pair_name> = pair name
            array of array entries(<time>, <open>, <high>, <low>, <close>, <vwap>, <volume>, <count>)
            last = id to be used as since when polling for new, committed OHLC data
    """
    params = {}
    if pair:
        params['pair'] = commasep(pair)
    else:
        raise pykraken.exceptions.BadParamterError()
    if interval:
        params['interval'] = interval
    if since:
        params['since'] = since
    c = client._post("/0/public/OHLC", params)
    return c['result']


def kpublic_depth(client, pair=None, count=None):
    """
    Returns array of pair name and market depth
    :param client: the client
    :param pair: asset pair to get market depth for
    :param count: maximum number of asks/bids (optional)
    :return: <pair_name> = pair name
                asks = ask side array of array entries(<price>, <volume>, <timestamp>)
                bids = bid side array of array entries(<price>, <volume>, <timestamp>)
    """
    params = {}
    if pair:
        params['pair'] = commasep(pair)
    else:
        raise pykraken.exceptions.BadParamterError()
    if count:
        params['count'] = count

    c = client._post("/0/public/Depth", params)
    return c['result']


def kpublic_trades(client, pair=None, since=None):
    """
    Returns an array of pair name and recent trade data
    :param client: the client
    :param pair: asset pair to get trade data for
    :param since: return trade data since given id (optional.  exclusive)
    :return: <pair_name> = pair name
        array of array entries(<price>, <volume>, <time>, <buy/sell>, <market/limit>, <miscellaneous>)
        last = id to be used as since when polling for new trade data
    """
    params = {}
    if pair:
        params['pair'] = commasep(pair)
    else:
        raise pykraken.exceptions.BadParamterError()
    if since:
        params['count'] = since

    c = client._post("/0/public/Trades", params)
    return c['result']


def kpublic_spread(client, pair=None, since=None):
    """
    Returns array of pair name and recent spread data
    :param client: the client
    :param pair: asset pair to get spread data for
    :param since: return spread data since given id (optional.  inclusive)
    :return: <pair_name> = pair name
        array of array entries(<time>, <bid>, <ask>)
        last = id to be used as since when polling for new spread data
    """
    params = {}
    if pair:
        params['pair'] = commasep(pair)
    else:
        raise pykraken.exceptions.BadParamterError()
    if since:
        params['count'] = since

    c = client._post("/0/public/Spread", params)
    return c['result']



import re

from .exceptions import BadParamterError


def commasep(entryList, sep=','):
    return sep.join(entryList)

def parseOTime(timestring):

    a = re.match('^\+\d+', timestring )
    b = re.match('^\d+', timestring)
    if a or b:
        return
    else:
        raise BadParamterError('+<n> = schedule start time <n> seconds from now <n> = unix timestamp of start time')

